﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;
using System.ServiceModel.Discovery;
using System.ServiceModel;
using Client.ServiceProxy;

// Demo by Raffaele Rialdi. May 20, 2010
// http://www.iamraf.net
// malta@vevy.com


namespace Client
{
	class Program
	{
		static void Main(string[] args)
		{
			var announcementService = new AnnouncementService();
			announcementService.OnlineAnnouncementReceived += new EventHandler<AnnouncementEventArgs>(OnlineAnnouncementReceived);
			announcementService.OfflineAnnouncementReceived += new EventHandler<AnnouncementEventArgs>(OfflineAnnouncementReceived);

			using(var announcementHost = new ServiceHost(announcementService))
			{
				announcementHost.AddServiceEndpoint(new UdpAnnouncementEndpoint());
				announcementHost.Open();

				FindExistingServices();

				Console.WriteLine("s start new service instance\r\n[num] to call instance\r\nq to exit");

				char ch;
				while((ch = GetConsoleChar()) != 'q')
				{
					switch(ch)
					{
						case 's':
							StartService();
							break;
						default:
							if(Char.IsNumber(ch))
							{
								int number = Int32.Parse(ch.ToString());
								string address;
								if(Instances.TryGetValue(number, out address))
								{
									CallInstance(address);
								}
							}
							break;
					}
				}
			}


		}


		private static void CallInstance(string address)
		{
			using(CalculatorClient client = new CalculatorClient("WSHttpBinding_ICalculator", address))
			{
				client.Sum(5, 3);
			}
		}

		private static char GetConsoleChar()
		{
			char c = Console.ReadKey(true).KeyChar;
			return Char.ToLower(c);
		}

		private static void StartService()
		{
			Process.Start(@"..\..\..\Service\bin\debug\Service.exe");
		}


		private static void FindExistingServices()
		{
			DiscoveryClient client = new DiscoveryClient(new UdpDiscoveryEndpoint());
			FindCriteria criteria = new FindCriteria(typeof(ICalculator));
			FindResponse response = client.Find(criteria);
			foreach(var endpoint in response.Endpoints)
			{
				Instances.Add(id, endpoint.Address.ToString());
				Console.WriteLine("{0} existing service: {1} Contract: ICalculator",
								id++,
								endpoint.Address.Uri.LocalPath
								);
			}
		}

		static Dictionary<int, string> Instances = new Dictionary<int, string>();
		static int id = 1;

		static void OnlineAnnouncementReceived(object sender, AnnouncementEventArgs e)
		{
			Instances.Add(id, e.EndpointDiscoveryMetadata.Address.ToString());

			Console.WriteLine("{0} service up: {1} Contract: {2}",
				id++,
				e.EndpointDiscoveryMetadata.Address.Uri.LocalPath,
				string.Join(";", 
					(from c in e.EndpointDiscoveryMetadata.ContractTypeNames select c.Name).ToArray())
				);

		}

		static void OfflineAnnouncementReceived(object sender, AnnouncementEventArgs e)
		{
			var id = (from i in Instances.Keys
					  where Instances[i] == e.EndpointDiscoveryMetadata.Address.ToString()
					  select i).First();
			Instances.Remove(id);

			Console.WriteLine("{0} service down: {1} Contract: {2}",
				id,
				e.EndpointDiscoveryMetadata.Address.Uri.LocalPath,
				string.Join(";",
					(from c in e.EndpointDiscoveryMetadata.ContractTypeNames select c.Name).ToArray())
				);

		}


	}
}
