﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;
using System.Windows;

// Demo by Raffaele Rialdi. May 20, 2010
// http://www.iamraf.net
// malta@vevy.com


namespace Vevy.Converters
{
	public class VisibilityConverter : IValueConverter
	{
		#region IValueConverter Members

		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Convert(value);
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return Convert(value);
		}

		private object Convert(object value)
		{
			if (value is bool)
			{
				bool b = (bool)value;
				return BoolToVisibility(b);
			}

			if (value is Visibility)
			{
				Visibility v = (Visibility)value;
				return VisibilityToBool(v);
			}

			throw new ArgumentException("Type " + value.GetType().ToString() + " not allowed in this converter");
		}

		protected virtual bool VisibilityToBool(Visibility value)
		{
			switch (value)
			{
				case Visibility.Hidden:
				case Visibility.Collapsed:
					return false;
				default:
					return true;
			}
		}

		protected virtual Visibility BoolToVisibility(bool value)
		{
			if (value)
				return Visibility.Collapsed;
			return Visibility.Visible;
		}

		#endregion
	}

	public class InvertedVisibilityConverter : VisibilityConverter
	{
		protected override Visibility BoolToVisibility(bool value)
		{
			return base.BoolToVisibility(!value);
		}

		protected override bool VisibilityToBool(Visibility value)
		{
			return base.VisibilityToBool(value);
		}
	}
}
