﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Collections.ObjectModel;
using System.Windows.Threading;
using System.ComponentModel;

// Demo by Raffaele Rialdi. May 20, 2010
// http://www.iamraf.net
// malta@vevy.com


namespace PowerClient
{
	public sealed class MainViewModel : BaseViewModel
	{
		public bool IsDesignMode { get; private set; }
		private Dispatcher Dispatcher { get; set; }

		public MainViewModel(Dispatcher dispatcher, bool isDesignMode)
		{
			this.Dispatcher = dispatcher;
			this.IsDesignMode = IsDesignMode;

			LastOperation = new Operation();
			LastOperation.PropertyChanged += (s, e) => { OnPropertyChanged("LastOperation"); };
			RandomizeOperandsAction();

			StartService = new DelegateCommand(StartServiceAction);
			CallService = new DelegateCommand<ServiceInfo>(CallServiceAction);
			RandomizeOperands = new DelegateCommand(RandomizeOperandsAction);
			OperatorChanged = new DelegateCommand(OperatorChangedAction);
		}

		internal void Load()
		{
			Services = new ObservableCollection<ServiceInfo>();

			#region Design time data
			if(this.IsDesignMode)
			{
				Services.Add(new ServiceInfo()
				{
					ContractName = "ITest",
					Address = new Uri("http://iamraf.net/xyz/1", UriKind.Absolute),
				});

				Services.Add(new ServiceInfo()
				{
					ContractName = "ITest",
					Address = new Uri("http://iamraf.net/xyz/2", UriKind.Absolute),
				});
			}
			#endregion

			ServiceDataLayer.Instance.NewServiceEvent += new EventHandler<NewServiceEventArgs>(OnNewService);
			ServiceDataLayer.Instance.ServiceShutdownEvent += new EventHandler<ServiceShutdownEventArgs>(OnServiceShutdown);

			ServiceDataLayer.Instance.FindExistingServicesOfICalculator();
		}

		void OnNewService(object sender, NewServiceEventArgs e)
		{
			Dispatcher.Invoke(() => Services.Add(e.ServiceInfo));
		}

		void OnServiceShutdown(object sender, ServiceShutdownEventArgs e)
		{
			Dispatcher.Invoke(() =>
				{
					var info = (from s in Services
								where s.Address == e.Address
								select s).FirstOrDefault();
					if(info != null)
						Services.Remove(info);
				});
		}


		private ObservableCollection<ServiceInfo> _Services;
		public ObservableCollection<ServiceInfo> Services
		{
			get { return _Services; }
			private set
			{
				_Services = value;
				this.OnPropertyChanged("Services");
			}
		}

		public ICommand StartService { get; private set; }
		private void StartServiceAction()
		{
			ServiceDataLayer.Instance.StartNewService();
		}

		public ICommand CallService { get; private set; }
		private void CallServiceAction(ServiceInfo info)
		{
			ServiceDataLayer.Instance.CallService(info, LastOperation);
		}


		public ICommand RandomizeOperands { get; private set; }
		private void RandomizeOperandsAction()
		{
			var rnd = new Random((int)DateTime.Now.Ticks);
			LastOperation.Operand1 = rnd.Next(1, 9);
			LastOperation.Operand2 = rnd.Next(1, 9);
			LastOperation.IsValid = false;
		}

		public ICommand OperatorChanged { get; private set; }
		private void OperatorChangedAction()
		{
			LastOperation.IsValid = false;
		}
	
		private Operation _LastOperation;
		public Operation LastOperation
		{
			get { return _LastOperation; }
			set
			{
				_LastOperation = value;
				OnPropertyChanged("LastOperation");
			}
		}


	}


}
