﻿//#define GENERATEMETADATA
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Discovery;
using System.ServiceModel.Description;
using System.Runtime.InteropServices;

// Demo by Raffaele Rialdi. May 20, 2010
// http://www.iamraf.net
// malta@vevy.com

namespace Service
{
	class Program
	{
		static void Main(string[] args)
		{
			try
			{
				Uri address = new Uri("http://localhost:8000/Calculator/" + (Guid.NewGuid()).ToString());
				using(var host = new ServiceHost(typeof(CalculatorV1), address))
				{
					ConsoleCloseHelper.SetHost(host);

					host.AddServiceEndpoint(typeof(ICalculator), new WSHttpBinding(), address);
#if GENERATEMETADATA
				var smb = new ServiceMetadataBehavior() {HttpGetEnabled = true};
				host.Description.Behaviors.Add(smb);
				host.AddServiceEndpoint(typeof(IMetadataExchange), new WSHttpBinding(), address.AbsolutePath + "/mex");
#endif

					var discoveryBehavior = new ServiceDiscoveryBehavior();
					discoveryBehavior.AnnouncementEndpoints.Add(new UdpAnnouncementEndpoint());

					host.Description.Behaviors.Add(discoveryBehavior);
					host.AddServiceEndpoint(new UdpDiscoveryEndpoint());
					host.Open();

					Console.WriteLine("Listening to:");
					Console.WriteLine(host.BaseAddresses[0].AbsoluteUri);

					Console.WriteLine("Service is ready ... <ENTER> to exit");
					Console.ReadLine();
				}
			}
			catch(Exception err)
			{
				Console.WriteLine(err.GetType().Name + ": " + err.Message);
				Console.WriteLine("");
				Console.WriteLine("If this was a CommunicationObjectFaultedException, you have two changes:");
				Console.WriteLine("1. run the service using the admin account");
				Console.WriteLine("2. assign the ACL to http://localhost:8000/ for your account");
				Console.WriteLine("");
				Console.WriteLine("For the option 2 you can this command from an admin command prompt:");
				Console.WriteLine(@"httpnamespace -a http://+:8000/ BUILTIN\Users");
				Console.WriteLine("HttpNamespace utility can be found at:");
				Console.WriteLine("http://www.iamraf.net/Tools/HttpNamespace-a-Win32-utility-to-reserve-an-HttpEndpoint");
				Console.ReadLine();
			}
		}


	}
}
